from collections import ChainMap
from pathlib import Path, PureWindowsPath

import arrow

from wap import __version__, log
from wap.config import TocConfig
from wap.exception import TocException
from wap.wowversion import WoWVersion

# tag names from https://wowwiki-archive.fandom.com/wiki/TOC_format
_OFFICIAL_TAGS = {
    "Interface",
    "Title",
    "Author",
    "Version",
    "Notes",
    "RequiredDeps",  # according to docs, this and the next have the same meaning to wow
    "Dependencies",
    "OptionalDeps",
    "LoadOnDemand",
    "LoadWith",
    "LoadManagers",
    "SavedVariables",
    "DefaultState",
    "Secure",
}

# we will warn about tags not in _OFFICIAL_TAGS that don't have the right prefix
_METADATA_TAG_PREFIX = "X-"


def write_toc(
    toc_config: TocConfig,
    dir_path: Path,
    write_path: Path,
    addon_version: str,
    wow_version: WoWVersion,
) -> None:
    for file in toc_config.files:
        rooted_file = write_path.parent / file
        if not rooted_file.is_file():
            raise TocException(
                f'TOC config lists file path "{file}", but it is not a file. This path '
                "must point to a file, must be relative to the path in the dir "
                f'config ("{dir_path.resolve()}") and, if it is in a subdirectory, '
                'must only use forward slashes ("/").'
            )

    extra_wap_tags = {
        "Interface": wow_version.interface_version(),
        "Version": addon_version,
        f"{_METADATA_TAG_PREFIX}BuildDateTime": arrow.utcnow().isoformat(),
        f"{_METADATA_TAG_PREFIX}BuildTool": f"wap v{__version__}",
    }

    tag_map = ChainMap(toc_config.tags, extra_wap_tags)

    for key, value in extra_wap_tags.items():
        if key in toc_config.tags:
            log.warn(
                f"Overwriting wap-provided tag {key}={value} with "
                "{toc_config.tags[key]}"
            )

    for key, value in toc_config.tags.items():
        if key not in _OFFICIAL_TAGS and not key.startswith(_METADATA_TAG_PREFIX):
            log.warn(
                f"TOC user-specified tag {key} does not have "
                f"{_METADATA_TAG_PREFIX} prefix"
            )

    # TOC file use windows path separators
    windows_files = [PureWindowsPath(f) for f in toc_config.files]

    lines = [
        *[f"## {key}: {value}\n" for key, value in tag_map.items()],
        "\n",
        *[f"{str(file)}\n" for file in windows_files],
    ]

    with write_path.open("w") as toc_file:
        for line in lines:
            toc_file.write(line)



import json
from pathlib import Path

import click

from wap import addon
from wap.commands.common import addon_version_option, config_path_option, json_option
from wap.config import Config
from wap.util import delete_path


@click.command()
@config_path_option()
@addon_version_option()
@json_option()
def build(
    config_path: Path,
    addon_version: str,
    show_json: bool,
) -> None:
    """
    Build your addon.

    For each wow-version specified in your config, create a build directory and zip file
    containing your packaged addon in the dist/ directory (relative to your current
    directory). The contents of the build directory and zip file will be the same as the
    dirs you provide in your config and an additional generated TOC file.

    The format of the name of the build directory will be
    "<addon-name>-<addon-version>-<wow-version-type>". It is the same for the zip file,
    except a ".zip" extension is added.

    Each TOC file generated
    will have the same name as the directory (plus the .toc extension). If this file
    exists in your source directory, it will be overrwritten.

    The contents of the TOC file will be those as specified in your toc tags and toc
    files.

    Unless they are overriden in your config, the tags "Interface" and "Version" in the
    generated TOC will correspond to the WoW version and addon version being built,
    respectively. wap also adds "X-BuildDateTime" and "X-BuildTool" tags with
    appropriate values.
    """
    config = Config.from_path(config_path)

    output_map = {}

    for wow_version in config.wow_versions:
        try:
            build_path = addon.build_addon(
                config_path=config_path,
                addon_name=config.name,
                dir_configs=config.dir_configs,
                addon_version=addon_version,
                wow_version=wow_version,
            )

            zip_path = addon.zip_addon(
                addon_name=config.name,
                wow_version=wow_version,
                addon_version=addon_version,
            )

        except BaseException:
            # catch any exception, keyboard interrupts included, and delete the
            # build path because it may be incomplete. this cleans up the build
            # and helps users not use broken builds in other commands
            build_path = addon.get_build_path(
                addon_name=config.name,
                addon_version=addon_version,
                wow_version=wow_version,
            )
            zip_path = addon.get_zip_path(
                addon_name=config.name,
                addon_version=addon_version,
                wow_version=wow_version,
            )

            for path in [build_path, zip_path]:
                if path.exists():
                    delete_path(path)

            # propogate up
            raise

        output_map[wow_version.type()] = {
            "build_dir_path": str(build_path),
            "zip_file_path": str(zip_path),
        }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from collections import defaultdict
from pathlib import Path, PurePosixPath
from typing import Any, Generic, Mapping, Optional, Sequence, TypeVar, cast

import attr
import strictyaml
from strictyaml.validators import Validator

from wap.exception import (
    ConfigFileException,
    ConfigSchemaException,
    ConfigSemanticException,
    WoWVersionException,
)
from wap.wowversion import WoWVersion

_Type = TypeVar("_Type")
_YamlObjectType = TypeVar("_YamlObjectType")


@attr.s(kw_only=True, auto_attribs=True, order=False)
class YamlType(Generic[_Type, _YamlObjectType], metaclass=ABCMeta):
    @classmethod
    def from_yaml(cls, yaml: str, label: str = "string") -> _Type:
        """
        label improves strictyaml's error messaging with the source of the yaml. if a
        file path is available, use that.
        """
        try:
            obj = strictyaml.load(
                yaml_string=yaml, schema=cls._yaml_schema(), label=label
            )
        except strictyaml.YAMLValidationError as yve:
            raise ConfigSchemaException(str(yve))

        return cls.from_python_object(obj.data)

    @classmethod
    @abstractmethod
    def from_python_object(
        cls,
        obj: _YamlObjectType,
    ) -> _Type:
        raise NotImplementedError()

    def to_yaml(self) -> str:
        # don't try-except here. we have a bug if the schema validation fails and want
        # to see the stack trace.

        # cast here because mypy thinks this is an Any
        return cast(
            str,
            strictyaml.as_document(
                data=self.to_python_object(),
                schema=self._yaml_schema(),
            ).as_yaml(),
        )

    @abstractmethod
    def to_python_object(
        self,
    ) -> _YamlObjectType:
        raise NotImplementedError()

    @classmethod
    @abstractmethod
    def _yaml_schema(cls) -> strictyaml.Validator:
        raise NotImplementedError()


@attr.s(kw_only=True, auto_attribs=True, order=False)
class TocConfig(YamlType["TocConfig", Mapping[str, Any]]):
    tags: Mapping[str, str]
    files: Sequence[PurePosixPath] = attr.ib()

    @classmethod
    def _yaml_schema(cls) -> strictyaml.Validator:
        return strictyaml.Map(
            {
                "tags": strictyaml.MapPattern(
                    strictyaml.Str(),
                    strictyaml.Str(),
                ),
                "files": strictyaml.Seq(
                    strictyaml.Str(),
                ),
            }
        )

    @files.validator
    def _check_files_relative(
        self,
        attribute: attr.Attribute[Sequence[PurePosixPath]],
        value: Sequence[PurePosixPath],
    ) -> None:
        if any(path.is_absolute() for path in value):
            raise ConfigSemanticException(
                f'File path "{value}" in TOC config must be relative (not absolute)'
            )

    @classmethod
    def from_python_object(
        cls,
        obj: Mapping[str, Any],
    ) -> TocConfig:
        tags = obj["tags"]

        files = [PurePosixPath(file) for file in obj["files"]]

        return cls(
            tags=tags,
            files=files,
        )

    def to_python_object(
        self,
    ) -> Mapping[str, Any]:
        return {
            "tags": {key: value for key, value in self.tags.items()},
            "files": [str(file) for file in self.files],
        }


@attr.s(kw_only=True, auto_attribs=True, order=False)
class DirConfig(YamlType["DirConfig", Mapping[str, Any]]):
    path: PurePosixPath = attr.ib()
    toc_config: TocConfig

    @path.validator
    def _check_path_relative(
        self,
        attribute: attr.Attribute[PurePosixPath],
        value: PurePosixPath,
    ) -> None:
        if value.is_absolute():
            raise ConfigSemanticException(
                f'Directory path "{value}" in dirs config must be relative (not '
                "absolute)"
            )

    @classmethod
    def _yaml_schema(cls) -> strictyaml.Validator:
        return strictyaml.Map(
            {
                "path": strictyaml.Str(),
                "toc": TocConfig._yaml_schema(),
            }
        )

    @classmethod
    def from_python_object(
        cls,
        obj: Mapping[str, Any],
    ) -> DirConfig:
        path = PurePosixPath(obj["path"])

        toc_config = TocConfig.from_python_object(obj["toc"])

        return cls(
            path=path,
            toc_config=toc_config,
        )

    def to_python_object(
        self,
    ) -> Mapping[str, Any]:
        obj: dict[str, Any] = {"path": str(self.path)}
        if self.toc_config is not None:
            obj["toc"] = self.toc_config.to_python_object()
        return obj


@attr.s(kw_only=True, auto_attribs=True, order=False)
class CurseforgeConfig(YamlType["CurseforgeConfig", Mapping[str, Any]]):
    project_id: str
    changelog_path: Optional[PurePosixPath] = attr.ib()
    project_slug: str

    @classmethod
    def _yaml_schema(cls) -> Validator:
        return strictyaml.Map(
            {
                "project-id": strictyaml.Str(),
                "project-slug": strictyaml.Str(),
                strictyaml.Optional("changelog-file"): strictyaml.Str(),
            }
        )

    @changelog_path.validator
    def _check_path_relative(
        self,
        attribute: attr.Attribute[PurePosixPath],
        value: PurePosixPath,
    ) -> None:
        if value.is_absolute():
            raise ConfigSemanticException(
                f'Changelog path "{value}" in curseforge config must be relative '
                "(not absolute)"
            )

    @classmethod
    def from_python_object(
        cls,
        obj: Mapping[str, Any],
    ) -> CurseforgeConfig:
        project_id = obj["project-id"]

        project_slug = obj["project-slug"]

        changelog_path: Optional[PurePosixPath] = None
        if "changelog-file" in obj:
            changelog_path = PurePosixPath(obj["changelog-file"])

        return cls(
            project_id=project_id,
            changelog_path=changelog_path,
            project_slug=project_slug,
        )

    def to_python_object(
        self,
    ) -> Mapping[str, Any]:
        obj: dict[str, Any] = {
            "project-id": self.project_id,
            "project-slug": self.project_slug,
        }
        if self.changelog_path is not None:
            obj["changelog-file"] = str(self.changelog_path)
        return obj


@attr.s(kw_only=True, auto_attribs=True, order=False)
class Config(YamlType["Config", Mapping[str, Any]]):
    name: str
    wow_versions: Sequence[WoWVersion] = attr.ib()
    curseforge_config: Optional[CurseforgeConfig] = attr.ib(default=None)
    dir_configs: Sequence[DirConfig]

    @wow_versions.validator
    def _check_wow_versions_no_dupe_types(
        self,
        attribute: attr.Attribute[Sequence[WoWVersion]],
        value: Sequence[WoWVersion],
    ) -> None:
        type_versions = defaultdict(list)
        for wow_version in value:
            type_versions[wow_version.type()].append(wow_version.dot_version())

        for type, versions in type_versions.items():
            if len(versions) > 1:
                raise ConfigSemanticException(
                    f"There must be at most one {type} version. Found {versions}."
                )

    @classmethod
    def _yaml_schema(cls) -> strictyaml.Validator:
        return strictyaml.Map(
            {
                "name": strictyaml.Str(),
                "wow-versions": strictyaml.Seq(
                    strictyaml.Str(),
                ),
                strictyaml.Optional("curseforge"): CurseforgeConfig._yaml_schema(),
                "dirs": strictyaml.Seq(DirConfig._yaml_schema()),
            }
        )

    @classmethod
    def from_python_object(
        cls,
        obj: Mapping[str, Any],
    ) -> Config:
        name: str = obj["name"]

        try:
            wow_versions = [
                WoWVersion.from_dot_version(wow_version)
                for wow_version in obj["wow-versions"]
            ]
        except WoWVersionException as wve:
            raise ConfigSemanticException(wve.message)

        curseforge_config = None
        if "curseforge" in obj:
            curseforge_config = CurseforgeConfig.from_python_object(obj["curseforge"])

        dir_configs = [DirConfig.from_python_object(dir_) for dir_ in obj["dirs"]]
        if len({dir_config.path for dir_config in dir_configs}) < len(dir_configs):
            raise ConfigSemanticException(
                "Directory paths in dir configs must have unique paths"
            )

        return cls(
            name=name,
            wow_versions=wow_versions,
            curseforge_config=curseforge_config,
            dir_configs=dir_configs,
        )

    def to_python_object(
        self,
    ) -> Mapping[str, Any]:
        obj: dict[str, Any] = {
            "name": self.name,
            "wow-versions": [
                wow_version.dot_version() for wow_version in self.wow_versions
            ],
        }
        if self.curseforge_config is not None:
            obj["curseforge"] = self.curseforge_config.to_python_object()

        # we could hypothetically put this in the map literal above, but I want to
        # normalize order. note that order of key-value pairs in YAML is not recognized,
        # but strictyaml uses an ordered dict to hold them, so this is possible.
        obj["dirs"] = [dir_config.to_python_object() for dir_config in self.dir_configs]

        return obj

    @classmethod
    def from_path(cls, path: Path) -> Config:
        if not path.is_file():
            raise ConfigFileException(f'No such config file "{path}"')

        with path.open("r") as file:
            contents = file.read()

        return cls.from_yaml(yaml=contents, label=str(path))

    def to_path(self, path: Path) -> None:
        with path.open("w") as file:
            file.write(self.to_yaml())



from __future__ import annotations

import json
from typing import BinaryIO, ClassVar, Optional

import attr
import requests

from wap.changelog import Changelog
from wap.exception import CurseForgeAPIException


@attr.s(kw_only=True, order=False, auto_attribs=True)
class CurseForgeAPI:

    api_token: str
    _version_map_cache: Optional[dict[str, int]] = attr.ib(
        default=None, init=False, eq=False
    )

    _SESSION: ClassVar[requests.Session] = requests.Session()
    _TOKEN_HEADER_NAME: ClassVar[str] = "X-Api-Token"
    _UPLOADED_FILE_URL_TEMPLATE: ClassVar[
        str
    ] = "https://www.curseforge.com/wow/addons/{slug}/files/{file_id}"

    RELEASE_TYPES: ClassVar[set[str]] = {"alpha", "beta", "release"}
    VERSION_ENDPOINT_URL: ClassVar[str] = "https://wow.curseforge.com/api/game/versions"
    UPLOAD_ENDPOINT_URL_TEMPLATE: ClassVar[
        str
    ] = "https://wow.curseforge.com/api/projects/{project_id}/upload-file"

    def upload_addon_file(
        self,
        *,
        project_id: str,
        archive_file: BinaryIO,
        display_name: str,
        changelog: Changelog,
        wow_version_id: int,
        release_type: str,
        file_name: str,
    ) -> int:
        """
        Uploads an addon file to Curseforge's WoW addon index.
        """
        metadata = self._create_release_metadata(
            changelog_contents=changelog.contents,
            display_name=display_name,
            version_id=wow_version_id,
            release_type=release_type,
            changelog_type=changelog.type,
        )

        resp = self._SESSION.post(
            url=self.UPLOAD_ENDPOINT_URL_TEMPLATE.format(project_id=project_id),
            headers={self._TOKEN_HEADER_NAME: self.api_token},
            data={"metadata": metadata},
            files={"file": (file_name, archive_file)},
        )

        if resp.status_code != requests.codes.ok:
            raise CurseForgeAPIException(
                f"Response from CurseForge during file upload has error status code "
                f"{resp.status_code}. Response body: {resp.text}"
            )

        return resp.json()["id"]  # type: ignore

    def get_version_id(self, *, version: str) -> int:
        # Since it's probably very common to upload 2 addons (a retail and classic), we
        # cache this response.
        if self._version_map_cache is None:
            resp = self._SESSION.get(
                url=self.VERSION_ENDPOINT_URL,
                headers={self._TOKEN_HEADER_NAME: self.api_token},
            )

            if resp.status_code != requests.codes.ok:
                raise CurseForgeAPIException(
                    f"Response from CurseForge during version lookup has error status "
                    f"code {resp.status_code}. Response body: {resp.text}"
                )

            # this is a weird hoop, but the big wigs packager does the same thing:
            # hypothetically, the endpoint could return multiple versions that have the
            # same wow version, so just choose the one with the max id because it's
            # probably the latest.
            self._version_map_cache = {}
            for cf_version_obj in resp.json():
                name, id_ = cf_version_obj["name"], cf_version_obj["id"]
                if name not in self._version_map_cache:
                    self._version_map_cache[name] = id_
                else:
                    if self._version_map_cache[name] < id_:
                        self._version_map_cache[name] = id_

        if version not in self._version_map_cache:
            raise CurseForgeAPIException(
                f'Version "{version}" could not be found on CurseForge'
            )

        return self._version_map_cache[version]

    @classmethod
    def _create_release_metadata(
        cls,
        changelog_contents: str,
        changelog_type: str,
        display_name: str,
        version_id: int,
        release_type: str,
    ) -> str:
        return json.dumps(
            {
                "changelog": changelog_contents,
                "changelogType": changelog_type,
                "displayName": display_name,
                "gameVersions": [version_id],
                "releaseType": release_type,
            }
        )

    @classmethod
    def uploaded_file_url(cls, slug: str, file_id: int) -> str:
        return cls._UPLOADED_FILE_URL_TEMPLATE.format(
            slug=slug,
            file_id=file_id,
        )



import json
from pathlib import Path
from typing import Optional

import click

from wap import addon
from wap.commands.common import (
    WAP_WOW_ADDONS_PATH_ENVVAR_NAME,
    addon_version_option,
    config_path_option,
    json_option,
)
from wap.config import Config
from wap.exception import DevInstallException
from wap.util import DEFAULT_DARWIN_WOW_ADDONS_PATH, DEFAULT_WIN32_WOW_ADDONS_PATH
from wap.wowversion import WoWVersion


class WowAddonsPathType(click.ParamType):
    name = "WOW_ADDONS_PATH"

    def convert(
        self,
        value: str,
        param: Optional[click.Parameter],
        ctx: Optional[click.Context],
    ) -> Path:
        path = Path(value)
        if not path.is_dir():
            self.fail(f"WoW AddOns path {value} is not a directory", param, ctx)

        not_addons_dir_exc_str = (
            f"WoW AddOns path {path} does not look like a WoW addons directory"
        )

        try:
            *_, wow_part, type_part, interface_part, addons_part = path.parts
        except ValueError:
            self.fail(not_addons_dir_exc_str, param, ctx)

        if (
            wow_part != "World of Warcraft"
            or interface_part != "Interface"
            or addons_part != "AddOns"
        ):
            self.fail(not_addons_dir_exc_str, param, ctx)

        if type_part not in WoWVersion.ADDONS_PATH_TYPE_MAP.keys():
            self.fail(not_addons_dir_exc_str, param, ctx)

        return path


WOW_ADDONS_PATH_TYPE = WowAddonsPathType()


@click.command()
@config_path_option()
@addon_version_option()
@json_option()
@click.option(
    "-w",
    "--wow-addons-path",
    envvar=WAP_WOW_ADDONS_PATH_ENVVAR_NAME,
    required=True,
    type=WOW_ADDONS_PATH_TYPE,
    help=(
        "Your WoW addons path. May also be specified in the environment variable "
        "WAP_WOW_ADDONS_PATH."
    ),
)
def dev_install(
    config_path: Path,
    addon_version: str,
    wow_addons_path: Path,
    show_json: bool,
) -> None:
    f"""
    Install a built addon to the provided WoW addons directory. (wap build must have
    been run before this.)

    This command assists you in testing your addons quickly.

    wap is smart in determining which addon build to install (retail or classic). It
    looks at the components of the WoW addons directory path provided and chooses the
    appropriate one.

    The provided WoW addons directory must appear to be valid, or else wap will not
    perform the installation. This is to avoid data loss in unintended directories.
    The actual logic for this is to inspect the path components of the directory
    provided, which must end with the following in order:

        1. "World of Warcraft"
        2. either "_retail_" or "_classic_"
        3. "Interface"
        4. "AddOns"

    For example, "{DEFAULT_WIN32_WOW_ADDONS_PATH}" (Windows) or
    "{DEFAULT_DARWIN_WOW_ADDONS_PATH}" (macOS) are acceptable.

    If your addon's directories already exist in the WoW addons directory, they will
    first be deleted to ensure a clean install. Keep this in mind if you have somehow
    put important data in that directory.
    """
    config = Config.from_path(config_path)

    wow_addons_path_type = WoWVersion.addons_path_type(wow_addons_path.parts[-3])

    wow_version: Optional[WoWVersion] = None
    for wow_version_in_config in config.wow_versions:
        if wow_version_in_config.type() == wow_addons_path_type:
            wow_version = wow_version_in_config
            break
    else:
        raise DevInstallException(
            f'No build exists for WoW addons path "{wow_addons_path}" (which is a '
            f"{wow_addons_path_type} installation). "
        )

    output_map = {}

    dev_install_paths = addon.dev_install_addon(
        wow_addons_path=wow_addons_path,
        addon_name=config.name,
        addon_version=addon_version,
        wow_version=wow_version,
    )

    output_map[wow_version.type()] = {
        "installed_dir_paths": [str(path) for path in dev_install_paths],
    }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))



from __future__ import annotations

import re
from typing import ClassVar, Mapping

import attr

from wap.exception import WoWVersionException


@attr.s(kw_only=True, auto_attribs=True, order=True, frozen=True)
class WoWVersion:

    _CLASSIC: ClassVar[str] = "classic"
    _RETAIL: ClassVar[str] = "retail"
    _DOT_PATTERN: ClassVar[str] = r"^(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)$"
    _INTERFACE_PATTERN: ClassVar[
        str
    ] = r"^(?P<major>\d+)(?P<minor>\d{2})(?P<patch>\d{2})$"
    _CLASSIC_MAJOR_THRESHOLD: ClassVar[int] = 1
    ADDONS_PATH_TYPE_MAP: ClassVar[Mapping[str, str]] = {
        "_retail_": _RETAIL,
        "_classic_": _CLASSIC,
    }

    major: int
    minor: int
    patch: int

    @classmethod
    def from_dot_version(cls, version: str) -> WoWVersion:
        if not (match := re.match(cls._DOT_PATTERN, version)):
            raise WoWVersionException(
                'WoW versions must be of form "x.y.z" where x, y, and z are integers'
            )

        return WoWVersion(
            major=int(match["major"]),
            minor=int(match["minor"]),
            patch=int(match["patch"]),
        )

    def dot_version(self) -> str:
        return f"{self.major}.{self.minor}.{self.patch}"

    def interface_version(self) -> str:
        return f"{self.major}{self.minor:0>2}{self.patch:0>2}"

    def type(self) -> str:
        if self.major > self._CLASSIC_MAJOR_THRESHOLD:
            return self._RETAIL
        return self._CLASSIC

    @classmethod
    def addons_path_type(cls, type_part: str) -> str:
        return cls.ADDONS_PATH_TYPE_MAP[type_part]


LATEST_RETAIL_VERSION = WoWVersion.from_dot_version("9.0.2")
LATEST_CLASSIC_VERSION = WoWVersion.from_dot_version("1.13.6")



import json
from pathlib import Path

import click

from wap import log
from wap.commands.common import config_path_option, json_option
from wap.config import Config
from wap.exception import ConfigException


@click.command()
@config_path_option()
@json_option(help="Prints the config file in JSON format.")
def validate(
    config_path: Path,
    show_json: bool,
) -> None:
    """
    Validates a wap configuration file.

    An exit code of 0 means the validation was successful. Otherwise, the error
    encountered is displayed and the exit code is non-zero.

    Successful validation does not indicate that you can use all the wap commands. It
    merely means that there were no errors parsing it.
    """
    try:
        config = Config.from_path(config_path)

        log.info(f'✔️ "{config_path}" is valid')

        if show_json:
            click.echo(json.dumps(config.to_python_object(), indent=2))

    except ConfigException:
        log.error(f'❌ "{config_path}" is not valid')
        raise



from pathlib import Path

import click

from wap import log
from wap.commands.common import DEFAULT_CONFIG_PATH, PATH_TYPE
from wap.exception import QuickstartException
from wap.guided_config import DEFAULT_CHANGELOG_PATH, guide
from wap.util import default_wow_addons_path_for_system
from wap.wowversion import WoWVersion

LATEST_RETAIL_VERSION = WoWVersion.from_dot_version("9.0.2")


def write_changelog(path: Path, name: str) -> None:
    with path.open("w") as changelog_file:
        changelog_file.write(f"# {name} Changelog\n\n")
        changelog_file.write(
            "This file is used by *wap* when you upload to Curseforge. It should "
            "contain a record of changes over time to your project.\n\n"
        )
        changelog_file.write("## Example Version 0.0.1\n\n")
        changelog_file.write("- Added feature X\n")
        changelog_file.write("- Fixed bug Y\n")


def write_readme(path: Path, name: str) -> None:
    with path.open("w") as readme_file:
        readme_file.write(f"# {name}\n\n")
        readme_file.write(
            "(This file is not required to run *wap*. It is here as a suggestion to "
            "document your project.)\n\n"
        )
        readme_file.write(
            'This file is written in Markdown, "a lightweight and easy-to-use syntax '
            'for styling all forms of writing".\n\n'
        )
        readme_file.write(
            "You can learn more about Markdown at "
            "<https://guides.github.com/features/mastering-markdown/>.\n\n"
        )
        readme_file.write(
            "Otherwise, go ahead and describe your project, such as...\n\n"
        )
        readme_file.write("## Features\n\n")
        readme_file.write("- Gives you free gold!\n")
        readme_file.write("- Boosts your DPS!\n")
        readme_file.write(
            "- Did someone say Thunderfury, Blessed Blade of the Windseeker?\n"
        )


def write_lua_file(path: Path, name: str) -> None:
    with path.open("w") as lua_file:
        lua_file.write("-- Your code can go here.\n")
        lua_file.write("-- Here's something to get you started,\n")
        lua_file.write("-- but you can erase it if you wish.\n\n")
        lua_file.write(f'local title = GetAddOnMetadata("{name}", "Title")\n')
        lua_file.write(f'local version = GetAddOnMetadata("{name}", "Version")\n')
        lua_file.write('print(title .. " version " .. version .. " has loaded.")\n')


@click.command()
@click.argument(
    "project_dir_path",
    type=PATH_TYPE,
)
def quickstart(
    project_dir_path: Path,
) -> None:
    """
    Creates a new addon project directory at PROJECT_DIR_PATH with a default structure.
    PROJECT_DIR_PATH must not exist.

    This command is interactive, and will ask you some questions about your project.

    If you later decide you want to change something about how you answered these
    question, edit your config file.

    For example, running:

        wap quickstart MyAddon

    will create a directory structure like the following:

      \b
      MyAddon
      ├── MyAddon
      │   └── Init.lua
      ├── CHANGELOG.md
      ├── README.md
      └── .wap.yml
    """
    # the docstring above contains the \b escape. This prevents paragraph wrapping
    # https://click.palletsprojects.com/en/7.x/documentation/#preventing-rewrapping
    # i'm not sure why i had to adjust the indentation level to get it to line up
    # properly though

    if project_dir_path.exists():
        raise QuickstartException(
            f"{project_dir_path} exists. Choose a path that does not exist for your "
            "new project."
        )

    config = guide(project_dir_name=project_dir_path.resolve().name)
    project_name = config.name

    log.info(
        '\nCreating project directory at "'
        + click.style(f"{project_dir_path}", fg="green")
        + '"'
    )
    project_dir_path.mkdir(parents=True)
    config_path = project_dir_path / DEFAULT_CONFIG_PATH
    log.info(
        'Writing config file at "' + click.style(f"{config_path}", fg="green") + '"'
    )
    config.to_path(config_path)

    log.info(
        "Creating changelog file at '"
        + click.style(f"{DEFAULT_CHANGELOG_PATH}", fg="green")
        + '"'
    )
    write_changelog(project_dir_path / DEFAULT_CHANGELOG_PATH, project_name)

    readme_path = project_dir_path / "README.md"
    log.info('Creating readme at "' + click.style(f"{readme_path}", fg="green"))
    write_readme(readme_path, project_name)

    # guided config puts 1 starter lua file in the toc config's files.
    dir_config = config.dir_configs[0]
    starter_lua_file = dir_config.toc_config.files[0]
    lua_file = project_dir_path / dir_config.path / starter_lua_file
    log.info(
        "Creating starter lua file at '" + click.style(f"{lua_file}", fg="green") + '"'
    )
    lua_file.parent.mkdir()
    write_lua_file(lua_file, project_name)

    log.info(
        click.style(
            "\nProject created! You can now begin developing your project.\n",
            fg="green",
            bold=True,
        )
    )
    log.info(
        "After you `"
        + click.style(f'cd "{project_dir_path}"', fg="magenta")
        + "`, you can get started running some wap commands immediately, such as:"
    )
    log.info("  - " + click.style("wap build", fg="blue"))
    log.info(
        "  - "
        + click.style(
            (
                "wap dev-install --wow-addons-path "
                f'"{default_wow_addons_path_for_system()}"'
            ),
            fg="blue",
        )
    )
    if config.curseforge_config:
        log.info(
            "  - "
            + click.style(
                'wap upload --addon-version "dev" --curseforge-token "<your-token>"',
                fg="blue",
            )
        )

    log.info("\nHave fun!")



from pathlib import Path

import click

from wap import log
from wap.commands.common import DEFAULT_CONFIG_PATH, PATH_TYPE
from wap.exception import NewConfigException
from wap.guided_config import guide


@click.command()
@click.option(
    "-c",
    "--config-path",
    type=PATH_TYPE,
    default=str(DEFAULT_CONFIG_PATH),
    show_default=str(DEFAULT_CONFIG_PATH),
    help=("The path of the configuration file to create."),
)
def new_config(
    config_path: Path,
) -> None:
    """
    Create a new configuration file with some pre-filled data.

    To avoid data loss, this path must not exist.

    This command is interactive, and will ask you some questions about your project.

    This command targeted towards existing projects that want to start using wap or
    projects that want to migrate from another packager.

    More than likely, you will need to edit this configuration file to fit to your
    project. This just provides a starting point.
    """
    if config_path.exists():
        raise NewConfigException(
            'Path "'
            + click.style(f"{config_path}", fg="green")
            + '" exists. Aborting to avoid data loss.'
        )

    project_name = Path.cwd().name

    config = guide(project_dir_name=project_name)
    log.info('\nCreating config file at "' + click.style(f"{config_path}", fg="green"))
    log.info(click.style("Make sure to edit it to fit your project!", fg="yellow"))
    config.to_path(config_path)



"""
Main entrypoint to the module
"""


def main() -> None:
    # imports are done here to reduce the amount of time spent outside the try-except
    # block, where a user can potentially interrupt the program

    # exit code reference http://www.tldp.org/LDP/abs/html/exitcodes.html

    try:
        from wap.commands import base

        exit_code = base.main(standalone_mode=False)

    # unfortunately, BaseException is the only exception superclass of KeyboardInterrupt
    except BaseException as exc:
        from click import Abort, ClickException

        from wap import log
        from wap.exception import WAPException

        if isinstance(exc, KeyboardInterrupt) or isinstance(exc, Abort):
            log.error("Interrupted")
            exit_code = 130

        elif isinstance(exc, WAPException):
            log.error(exc.message)
            exit_code = 1

        elif isinstance(exc, ClickException):
            log.error(exc.format_message())
            exit_code = 1

        else:
            raise

    import sys

    sys.exit(exit_code)


if __name__ == "__main__":
    main()



class WAPException(Exception):
    """General WAP exception"""

    def __init__(self, message: str):
        self.message = message


class ConfigException(WAPException):
    """Indicates a general problem with the config."""


class ConfigSemanticException(ConfigException):
    """Indicates there was a problem with the data in the config"""


class ConfigSchemaException(ConfigException):
    """Indicates the loaded yaml does not follow the yaml schema"""


class ConfigFileException(ConfigException):
    """Indicates the config file could not be found"""


class WoWVersionException(WAPException):
    """Indicates a problem creating a version"""


class DevInstallException(WAPException):
    """Indicates a problem during dev-installing"""


class BuildException(WAPException):
    """Indicates a problem during building"""


class TocException(WAPException):
    """Indicates a problem with a TOC file"""


class UploadException(WAPException):
    """Indicates a problem during uploading"""


class CurseForgeAPIException(WAPException):
    """Indicates a problem communicating with CurseForge"""


class QuickstartException(WAPException):
    """Indicates a problem creating a new project"""


class NewConfigException(WAPException):
    """Indicates a problem creating a new configuration file"""


class ChangelogException(WAPException):
    """Indicates a problem building a changelog"""



from functools import update_wrapper
from pathlib import Path
from typing import Any, Callable, Optional, TypeVar

import click


class PathType(click.ParamType):
    name = "PATH"

    def convert(
        self,
        value: str,
        param: Optional[click.Parameter],
        ctx: Optional[click.Context],
    ) -> Path:
        return Path(value)


PATH_TYPE = PathType()

VERSION_STRING_TEMPLATE = "wap, version %(version)s"

OUTPUT_PATH = Path("dist")

DEFAULT_CONFIG_PATH = Path(".wap.yml")
DEFAULT_ADDON_VERSION = "dev"
DEFAULT_RELEASE_TYPE = "alpha"

WAP_CURSEFORGE_TOKEN_ENVVAR_NAME = "WAP_CURSEFORGE_TOKEN"
WAP_CONFIG_PATH_ENVVAR_NAME = "WAP_CONFIG_PATH"
WAP_WOW_ADDONS_PATH_ENVVAR_NAME = "WAP_WOW_ADDONS_PATH"

_DECORATED_FUNC_TYPE = TypeVar("_DECORATED_FUNC_TYPE", bound=Callable[..., Any])


def config_path_option() -> Callable[[_DECORATED_FUNC_TYPE], _DECORATED_FUNC_TYPE]:
    def wrapper(func: _DECORATED_FUNC_TYPE) -> _DECORATED_FUNC_TYPE:
        decorated = click.option(
            "-c",
            "--config-path",
            type=PATH_TYPE,
            default=str(DEFAULT_CONFIG_PATH),
            envvar=WAP_CONFIG_PATH_ENVVAR_NAME,
            show_default=str(DEFAULT_CONFIG_PATH),
            help=(
                "The path of the configuration file. May also be specified in the "
                "environment variable WAP_CONFIG_PATH."
            ),
        )(func)

        return update_wrapper(decorated, func)

    return wrapper


def addon_version_option(
    *, required: bool = False
) -> Callable[[_DECORATED_FUNC_TYPE], _DECORATED_FUNC_TYPE]:
    def wrapper(func: _DECORATED_FUNC_TYPE) -> _DECORATED_FUNC_TYPE:
        decorated = click.option(
            "-v",
            "--addon-version",
            required=required,
            default=None if required else DEFAULT_ADDON_VERSION,
            show_default=not required,
            help="The developer-defined version of your addon package.",
        )(func)

        return update_wrapper(decorated, func)

    return wrapper


def json_option(
    *, help: Optional[str] = None
) -> Callable[[_DECORATED_FUNC_TYPE], _DECORATED_FUNC_TYPE]:
    if help is None:
        help = (
            "Output json to stdout of the operations wap performed (so it can be "
            "written to files or piped to other programs)"
        )

    def wrapper(func: _DECORATED_FUNC_TYPE) -> _DECORATED_FUNC_TYPE:
        decorated = click.option(
            "-j",
            "--json",
            "show_json",
            is_flag=True,
            default=False,
            help=help,
        )(func)

        return update_wrapper(decorated, func)

    return wrapper



import click

from wap import __version__, log
from wap.commands.build import build
from wap.commands.common import VERSION_STRING_TEMPLATE
from wap.commands.dev_install import dev_install
from wap.commands.new_config import new_config
from wap.commands.quickstart import quickstart
from wap.commands.upload import upload
from wap.commands.validate import validate


@click.group()
@click.version_option(
    version=__version__,
    message=VERSION_STRING_TEMPLATE,
)
def base() -> None:
    """Build and upload your WoW addons."""
    # always print out version info on run (will help with issue reports)
    log.info((VERSION_STRING_TEMPLATE + "\n") % {"version": __version__})


SUBCOMMANDS = [
    build,
    dev_install,
    upload,
    quickstart,
    new_config,
    validate,
]

for subcommand in SUBCOMMANDS:
    base.add_command(subcommand)



import shutil
import sys
from pathlib import Path

DEFAULT_WIN32_WOW_ADDONS_PATH = (
    R"C:\Program Files (x86)\World of Warcraft\_retail_\Interface\AddOns"
)
DEFAULT_DARWIN_WOW_ADDONS_PATH = (
    "/Applications/World of Warcraft/_retail_/Interface/AddOns"
)


def delete_path(path: Path) -> None:
    """
    Deletes either a file or directory. The path existance must be checked before
    calling this function.
    """
    if path.is_file():
        path.unlink()
    elif path.is_dir():
        shutil.rmtree(path)
    else:  # pragma: no cover
        # don't measure code coverage here because it's a really esoteric case:
        # it could be a pipe, link, socket or some other non-file thing.
        raise ValueError(
            f"Cannot delete path {path} because it is not a file or directory"
        )


def default_wow_addons_path_for_system() -> str:
    # this function is for documentation purposes only! we will not start assuming
    # where a user's addons path.
    platform = sys.platform

    # don't measure code coverage here because the testing system is only one platform
    # TODO: write unit test that mocks out sys.platform?
    if platform == "win32":  # pragma: no cover
        return DEFAULT_WIN32_WOW_ADDONS_PATH
    elif platform == "darwin":  # pragma: no cover
        return DEFAULT_DARWIN_WOW_ADDONS_PATH
    else:  # pragma: no cover
        return "</path/to/WoW/Interface/AddOns>"



from __future__ import annotations

from pathlib import Path

import attr

from wap import log

CHANGELOG_SUFFIX_MAP = {
    ".md": "markdown",
    ".markdown": "markdown",
    ".html": "html",
    ".txt": "text",
}

CHANGELOG_TYPES = {t for t in CHANGELOG_SUFFIX_MAP.values()}


@attr.s(auto_attribs=True, frozen=True, order=False, kw_only=True)
class Changelog:
    contents: str
    type: str = attr.ib()

    @type.validator
    def _check_type_valid(
        self,
        attribute: attr.Attribute[str],
        value: str,
    ) -> None:
        # no code coverage here: we either get this from the command line, which
        # requires the type to be from the CHANGELOG_TYPES, or from from_path(), which
        # will guess at the type, and fallback to text if unknown.
        #
        # however, if we somehow forget this and create the object willy-nilly,
        # this ValueError will leak through and stacktrace, which we want.
        if value not in CHANGELOG_TYPES:  # pragma: no cover
            raise ValueError(f"Changelog type {value} must be in {CHANGELOG_TYPES}")

    @classmethod
    def from_path(cls, path: Path) -> Changelog:
        normalized_changelog_suffix = path.suffix.lower()
        if normalized_changelog_suffix in CHANGELOG_SUFFIX_MAP:
            type = CHANGELOG_SUFFIX_MAP[normalized_changelog_suffix]
        else:
            log.warn(
                f"Unable to determine changelog type from extension for {path}, "
                'so assuming "text"'
            )
            type = "text"

        with path.open("r") as file:
            contents = file.read()

        return cls(
            type=type,
            contents=contents,
        )



"""
Logging setup
"""

from __future__ import annotations

from typing import Optional

import click

# the python standard logging module is the most degenerate shit ever
#   - global state
#   - multiple loggers, handlers, formatters? overengineered
#   - strange filtering (loggers and handlers can have different levels)
#   - need to hack in ISO8601-compliant timestamps
#   - hard to test because different loggers, handlers, etc
#   - warn vs warning functions (not to even mention the additional warnings module)
#   - it uses javaCasing, not python_casing
#
# let's just make some functions that do what we want. this will also allow further
# improvements, like color, in the future

# some logging lines are not covered by code coverage here because they are not
# exercised by wap currently. I'd rather retain the full set of logging functions
# in case we need them in the future. Hopefully, their simplicity should be self-evident


def _log(*, label: Optional[str] = None, msg: str) -> None:
    out = msg
    if label is not None:
        out = f"{label} - {msg}"
    click.echo(out, err=True)


def debug(msg: str) -> None:
    _log(label=click.style("DEBUG", fg="cyan"), msg=msg)  # pragma: no cover


def info(msg: str) -> None:
    # no label here. this is the default way we should be communicating with the user
    _log(msg=msg)  # pragma: no cover


def warn(msg: str) -> None:
    _log(label=click.style("WARN", fg="yellow"), msg=msg)  # pragma: no cover


def error(msg: str) -> None:
    _log(label=click.style("ERROR", fg="red"), msg=msg)  # pragma: no cover



import shutil
from collections.abc import Sequence
from pathlib import Path
from zipfile import ZIP_DEFLATED, ZipFile

import click

from wap import log
from wap.changelog import Changelog
from wap.config import CurseforgeConfig, DirConfig
from wap.curseforge import CurseForgeAPI
from wap.exception import BuildException, DevInstallException, UploadException
from wap.toc import write_toc
from wap.util import delete_path
from wap.wowversion import WoWVersion


def get_output_path() -> Path:
    return Path("dist")


def get_build_path(
    addon_name: str, addon_version: str, wow_version: WoWVersion
) -> Path:
    return get_output_path() / f"{addon_name}-{addon_version}-{wow_version.type()}"


def get_zip_path(addon_name: str, addon_version: str, wow_version: WoWVersion) -> Path:
    bp = get_build_path(
        addon_name=addon_name,
        addon_version=addon_version,
        wow_version=wow_version,
    )
    return bp.with_name(bp.name + ".zip")


def build_addon(
    config_path: Path,
    addon_name: str,
    dir_configs: Sequence[DirConfig],
    addon_version: str,
    wow_version: WoWVersion,
) -> Path:
    build_path = get_build_path(
        addon_name=addon_name,
        addon_version=addon_version,
        wow_version=wow_version,
    )

    if build_path.exists():
        delete_path(build_path)

    build_path.mkdir(parents=True)

    for dir_config in dir_configs:
        src_dir = config_path.parent / dir_config.path

        if not src_dir.is_dir():
            raise BuildException(
                f'Dir config has path "{src_dir}", but it is not a directory. '
                "This path must point to a directory, must be relative to "
                f'the parent of the config file ("{config_path.resolve()}") and, '
                'if it is in a subdirectory, must only use forward slashes ("/").'
            )

        dst_dir = build_path / src_dir.name

        shutil.copytree(src_dir, dst_dir)

        source_toc_path = src_dir / (src_dir.name + ".toc")
        toc_path = dst_dir / (src_dir.name + ".toc")

        if toc_path.exists():
            log.warn(
                f'TOC file "{source_toc_path}" exists, and will '
                "be overwritten with a generated one."
            )
            delete_path(toc_path)

        write_toc(
            toc_config=dir_config.toc_config,
            dir_path=src_dir,
            write_path=toc_path,
            addon_version=addon_version,
            wow_version=wow_version,
        )

    log.info(
        "Built addon "
        + click.style(f"{addon_name}", fg="blue")
        + " ("
        + click.style(f"{wow_version.type()}", fg="magenta")
        + ') at "'
        + click.style(f"{build_path}", fg="green")
        + '"'
    )

    return build_path


def zip_addon(addon_name: str, addon_version: str, wow_version: WoWVersion) -> Path:
    build_path = get_build_path(
        addon_name=addon_name, addon_version=addon_version, wow_version=wow_version
    )
    zip_path = get_zip_path(
        addon_name=addon_name, addon_version=addon_version, wow_version=wow_version
    )

    if zip_path.exists():
        delete_path(zip_path)

    zip_path.parent.mkdir(parents=True, exist_ok=True)

    with ZipFile(zip_path, mode="w", compression=ZIP_DEFLATED) as zip_file:
        for path in build_path.rglob("*"):
            zip_file.write(
                filename=path,
                arcname=path.relative_to(build_path),
            )

    log.info(
        "Zipped addon "
        + click.style(f"{addon_name}", fg="blue")
        + " ("
        + click.style(f"{wow_version.type()}", fg="magenta")
        + ') at "'
        + click.style(f"{zip_path}", fg="green")
        + '"'
    )

    return zip_path


def upload_addon(
    *,
    addon_name: str,
    curseforge_config: CurseforgeConfig,
    changelog: Changelog,
    wow_version: WoWVersion,
    addon_version: str,
    release_type: str,
    curseforge_api: CurseForgeAPI,
) -> str:
    cf_wow_version_id = curseforge_api.get_version_id(version=wow_version.dot_version())

    zip_file_path = get_zip_path(
        addon_name=addon_name, addon_version=addon_version, wow_version=wow_version
    )

    if not zip_file_path.is_file():
        log.error(
            "Expected zip file not found. Have you run `"
            + click.style(f'wap build --addon-version "{addon_version}"', fg="blue")
            + "` yet?"
        )
        raise UploadException(f'Zip file "{zip_file_path}" not found.')

    with zip_file_path.open("rb") as package_archive_file:
        file_id = curseforge_api.upload_addon_file(
            project_id=curseforge_config.project_id,
            archive_file=package_archive_file,
            display_name=f"{addon_name}-{addon_version}-{wow_version.type()}",
            changelog=changelog,
            wow_version_id=cf_wow_version_id,
            release_type=release_type,
            file_name=zip_file_path.name,
        )

    url = curseforge_api.uploaded_file_url(
        slug=curseforge_config.project_slug,
        file_id=file_id,
    )
    log.info(
        "Uploaded "
        + click.style(f"{addon_name}", fg="blue")
        + " ("
        + click.style(f"{wow_version.type()}", fg="magenta")
        + ") to CurseForge at "
        + click.style(f"{url}", fg="green")
    )

    return url


def dev_install_addon(
    *,
    addon_name: str,
    addon_version: str,
    wow_addons_path: Path,
    wow_version: WoWVersion,
) -> Sequence[Path]:
    installed_paths = []

    build_path = get_build_path(
        addon_name=addon_name, addon_version=addon_version, wow_version=wow_version
    )

    if not build_path.is_dir():
        log.error(
            "Expected build directory not found. Have you run `"
            + click.style(f"wap build --addon-version {addon_version}", fg="blue")
            + "` yet?"
        )
        raise DevInstallException(f'Build directory "{build_path}" not found.')

    for addon_path in build_path.iterdir():
        install_addon_path = wow_addons_path / addon_path.name

        if install_addon_path.exists():
            delete_path(install_addon_path)

        shutil.copytree(addon_path, install_addon_path)

        installed_paths.append(install_addon_path)

        log.info(
            "Installed addon directory "
            + click.style(f"{addon_path.name}", fg="blue")
            + " ("
            + click.style(f"{wow_version.type()}", fg="magenta")
            + ') to "'
            + click.style(f"{install_addon_path}", fg="green")
            + '"'
        )

    return installed_paths



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wap', 'wap.commands']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=0.17.0,<0.18.0',
 'attrs>=20.3.0,<21.0.0',
 'click>=7.1.2,<8.0.0',
 'colorama>=0.4.4,<0.5.0',
 'requests>=2.25.1,<3.0.0',
 'strictyaml>=1.3.2,<2.0.0']

entry_points = \
{'console_scripts': ['wap = wap.__main__:main']}

setup_kwargs = {
    'name': 'wow-addon-packager',
    'version': '0.7.0',
    'description': 'A user-friendly World of Warcraft addon packager',
    'long_description': 'wap (WoW Addon Packager)\n========================\n\n.. teaser-begin\n\n.. image:: https://github.com/t-mart/wap/actions/workflows/ci.yml/badge.svg?branch=master\n   :target: https://github.com/t-mart/wap/actions/workflows/ci.yml\n   :alt: GitHub Actions status for master branch\n\n.. image:: https://codecov.io/gh/t-mart/wap/branch/master/graph/badge.svg?token=AVOA4QWTBL\n   :target: https://codecov.io/gh/t-mart/wap\n   :alt: Code Coverage on codecov.io\n\n.. image:: https://img.shields.io/pypi/v/wow-addon-packager\n   :target: https://pypi.org/project/wow-addon-packager/\n   :alt: Latest release on PyPI\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Code styled with black\n\n.. image:: https://img.shields.io/github/license/t-mart/wap\n   :target: https://github.com/t-mart/wap/blob/master/LICENSE\n   :alt: MIT licensed\n\n.. image:: https://readthedocs.org/projects/wow-addon-packager/badge/?version=latest\n   :target: https://wow-addon-packager.readthedocs.io/en/latest/?badge=latest\n   :alt: Documentation Status\n\n|\n\n``wap`` is a user-friendly World of Warcraft addon packager.\n\n.. image:: https://raw.githubusercontent.com/t-mart/wap/master/docs/_static/demo.gif\n   :alt: wap demo\n\nFeatures\n--------\n\n- Builds retail or classic WoW addons (or both!)\n- Uploads your addons to CurseForge\n- Installs your addons to your Addons folder for fast development feedback\n- Generates valid TOC files automagically\n- Sets up new addon projects quickly, ready to go with one command\n- Consolidates all configuration in one easy-to-edit file\n- Supports and is tested on Windows, macOS, and Linux\n- Has awesome `documentation`_\n\n.. _`documentation`: https://wow-addon-packager.readthedocs.io/en/stable\n\n.. teaser-end\n\n\n``wap`` in 5 minutes\n--------------------\n\n.. five-minutes-begin\n\nThis entire set of instructions is runnable without editing a single line of code!\n\n1. `Download Python 3.9 or greater`_ and install it.\n\n2. Install ``wap`` with pip:\n\n   .. code-block:: console\n\n      $ pip install --upgrade --user wow-addon-packager\n\n3. Create a new project:\n\n   .. code-block:: console\n\n      $ wap quickstart MyAddon  # or whatever name you\'d like!\n\n   and answer the prompted questions. Don\'t worry too much about your answers -- you can\n   always change them later in your configuration file.\n\n   Then change to your new project\'s directory\n\n   .. code-block:: console\n\n      $ cd "MyAddon"\n\n4. Build your addon\n\n   .. code-block:: console\n\n      $ wap build\n\n5. Install your addon so you can test it out in your local WoW installation:\n\n   .. code-block:: console\n\n      on Windows\n      $ wap dev-install --wow-addons-path "C:\\Program Files (x86)\\World of Warcraft\\_retail_\\Interface\\AddOns"\n\n      on macOS\n      $ wap dev-install --wow-addons-path "/Applications/World of Warcraft/_retail_/Interface/AddOns"\n\n6. Upload your project to CurseForge\n\n   .. code-block:: console\n\n      $ wap upload --addon-version "dev" --curseforge-token "<your-token>"\n\n   You can generate a new token at Curseforge\'s `My API Tokens`_ page.\n\n.. _`My API Tokens`: https://authors.curseforge.com/account/api-tokens\n.. _`Download Python 3.9 or greater`: https://www.python.org/downloads/\n\n.. five-minutes-end\n\nFurther Help\n------------\n\nSee the `official documentation site`_. There\'s a lot more information there!\n\nAlso, the ``wap`` command is fully documented in its help text. View it with:\n\n.. code-block:: console\n\n   $ wap --help\n   $ wap build --help\n   $ wap upload --help\n   ... etc\n\nContributing\n------------\n\nSee `how to contribute`_ in the official docs.\n\nTODOs\n-----\n\n- localization via curseforge?\n- Dockerfile github action `<https://docs.github.com/en/actions/creating-actions/creating-a-docker-container-action>`_\n\n.. _`how to contribute`: https://wow-addon-packager.readthedocs.io/en/stable/contributing.html\n.. _`official documentation site`: https://wow-addon-packager.readthedocs.io/en/stable\n',
    'author': 'Tim Martin',
    'author_email': 'tim@timmart.in',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/t-mart/wap',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import json
from pathlib import Path
from typing import Optional

import click

from wap import addon
from wap.changelog import CHANGELOG_TYPES, Changelog
from wap.commands.common import (
    DEFAULT_RELEASE_TYPE,
    WAP_CURSEFORGE_TOKEN_ENVVAR_NAME,
    addon_version_option,
    config_path_option,
    json_option,
)
from wap.config import Config
from wap.curseforge import CurseForgeAPI
from wap.exception import ChangelogException, UploadException


@click.command()
@config_path_option()
@json_option()
@addon_version_option(required=True)
@click.option(
    "-r",
    "--release-type",
    type=click.Choice(list(CurseForgeAPI.RELEASE_TYPES)),
    default=DEFAULT_RELEASE_TYPE,
    show_default=True,
    help="The type of release to make.",
)
@click.option(
    "--curseforge-token",
    envvar=WAP_CURSEFORGE_TOKEN_ENVVAR_NAME,
    required=True,
    help=(
        "The value of your CurseForge API token. May also be specified in the "
        "environment variable WAP_CURSEFORGE_TOKEN."
    ),
)
@click.option(
    "--changelog-contents",
    help=(
        "The contents of your changelog that will be displayed with your upload on "
        "CurseForge. If you have also provided a changelog-file in your config, "
        "this option will take precedence. There are no requirements for these "
        "contents -- they may be blank if you wish. Must be used in conjunction with "
        "--changelog-type."
    ),
)
@click.option(
    "--changelog-type",
    type=click.Choice(list(CHANGELOG_TYPES), case_sensitive=False),
    help=(
        "The format of your changelog contents. Must be used in conjunction with "
        "--changelog-contents."
    ),
)
@click.pass_context
def upload(
    ctx: click.Context,
    config_path: Path,
    addon_version: str,
    release_type: str,
    curseforge_token: str,
    show_json: bool,
    changelog_contents: Optional[str],
    changelog_type: Optional[str],
) -> None:
    """
    Upload built addons to your addons Curseforge page. (wap build must have been run
    before this.)

    Each build of your addon (retail and/or classic) with the given addon version will
    be uploaded. An addon version is **required** from you for this command. This is to
    ensure that your uploads are intentional, which are released to the Internet.

    In addition to the options set for this command and your configuration, wap
    automatically sets some metadata to send with the request.
        - The display name. This is the name of the file as it appears on your addon's
          files page. wap sets this to
          <addon-name>-<addon-version>-<wow-version-type>
        - The zip file name. This is the file name of the that users download. wap sets
          this to <addon-name>-<addon-version>-<wow-version-type>.zip
    """
    config = Config.from_path(config_path)

    curseforge_config = config.curseforge_config
    if curseforge_config is None:
        raise UploadException(
            'A "curseforge" configuration section must be provided in config to upload'
        )

    if (changelog_contents is None) ^ (changelog_type is None):
        raise ChangelogException(
            "--changelog-contents and --changelog-type must be used together or "
            "not at all."
        )
    else:

        if changelog_contents is not None:
            changelog = Changelog(
                contents=changelog_contents,
                type=changelog_type,  # type: ignore
            )
        elif curseforge_config.changelog_path is not None:
            changelog_path = config_path.parent / curseforge_config.changelog_path
            if not changelog_path.is_file():
                raise UploadException(
                    f'Curseforge config has changelog path "{changelog_path}", but it '
                    "is not a file. This path must point to a file, must be relative "
                    f'to the parent of the config file ("{config_path.resolve()}") '
                    "and, if it is in a subdirectory, must only use forward slashes "
                    '("/"). Or, you may use the --changelog-contents and '
                    "--changelog-type options "
                )
            changelog = Changelog.from_path(changelog_path)

    output_map = {}

    curseforge_api = CurseForgeAPI(api_token=curseforge_token)

    for wow_version in config.wow_versions:
        upload_url = addon.upload_addon(
            addon_name=config.name,
            curseforge_config=curseforge_config,
            changelog=changelog,
            wow_version=wow_version,
            addon_version=addon_version,
            release_type=release_type,
            curseforge_api=curseforge_api,
        )

        output_map[wow_version.type()] = {
            "curseforge_upload_url": upload_url,
        }

    if show_json:
        click.echo(json.dumps(output_map, indent=2))



import re
from pathlib import PurePosixPath
from typing import Any, Optional, Pattern, cast

import click

from wap import log
from wap.config import Config, CurseforgeConfig, DirConfig, TocConfig
from wap.wowversion import LATEST_CLASSIC_VERSION, LATEST_RETAIL_VERSION

DEFAULT_CHANGELOG_PATH = PurePosixPath("CHANGELOG.md")


def _prompt_until_matching(
    pattern: Pattern[str],
    text: str,
    default: Optional[str] = None,
    **prompt_kwargs: Any,
) -> re.Match[str]:
    text = click.style(text, fg="blue")
    if default is not None:
        text += " [" + click.style(default, fg="green") + "]"
    while True:
        value = cast(
            str,
            click.prompt(
                text, default=default, show_default=False, err=True, **prompt_kwargs
            ),
        )
        if (match := pattern.match(value)) is not None:
            return match
        log.info(f'Value "{value}" does not match pattern: {pattern.pattern}')


def _prompt_yes_no(text: str, **prompt_kwargs: Any) -> bool:
    yes_no_pattern = re.compile(r"^(y|(?:yes)|n|(?:no))$", flags=re.IGNORECASE)
    text = (
        click.style(text, fg="blue")
        + " <"
        + click.style("yes", fg="magenta")
        + "|"
        + click.style("y", fg="magenta")
        + "|"
        + click.style("no", fg="magenta")
        + "|"
        + click.style("n", fg="magenta")
        + ">"
    )
    while True:
        value = cast(str, click.prompt(text=text, err=True, **prompt_kwargs))
        if (match := yes_no_pattern.match(value)) is not None:
            return "y" in match[0].lower()
        log.info(f'Value "{value}" does not match pattern: {yes_no_pattern.pattern}')


def guide(project_dir_name: str) -> Config:
    addon_name_pattern = re.compile(r"\S+")
    author_pattern = re.compile(r".+")
    notes_pattern = re.compile(r".+")
    project_id_pattern = re.compile(r"\d+")
    curseforge_url_pattern = re.compile(
        r"https:\/\/www\.curseforge\.com\/wow\/addons\/(?P<addon_name>\S+)"
    )

    log.info(
        "This command will guide you through creating your "
        + click.style(".wap.yml", fg="blue")
        + " config.\n",
    )

    # addon name
    name = _prompt_until_matching(
        addon_name_pattern,
        text="Addon name",
        default=project_dir_name,
    )[0]

    # author for TOC
    author = _prompt_until_matching(
        author_pattern,
        text="Author name",
    )[0]

    # notes for TOC
    notes = _prompt_until_matching(
        notes_pattern,
        text="Addon description",
    )[0]

    toc_config = TocConfig(
        tags={"Title": name, "Author": author, "Notes": notes},
        files=[PurePosixPath("Init.lua")],
    )

    dir_configs = [DirConfig(path=PurePosixPath(name), toc_config=toc_config)]

    # wow versions supported
    wow_versions = []

    if _prompt_yes_no(text="Will this addon support retail WoW?"):
        wow_versions.append(LATEST_RETAIL_VERSION)
        if _prompt_yes_no(text="Will this addon support classic WoW?"):
            wow_versions.append(LATEST_CLASSIC_VERSION)
    else:
        wow_versions.append(LATEST_CLASSIC_VERSION)
        log.info("If not supporting retail, must be supporting classic")

    # curseforge
    curseforge_config = None
    if _prompt_yes_no(text="Do you have a CurseForge project?"):

        project_id = _prompt_until_matching(
            project_id_pattern,
            text="CurseForge project id (found in top-right of addon page)",
        )[0]

        project_slug = _prompt_until_matching(
            curseforge_url_pattern,
            text="CurseForge URL",
        )["addon_name"]

        curseforge_config = CurseforgeConfig(
            project_id=project_id,
            changelog_path=DEFAULT_CHANGELOG_PATH,
            project_slug=project_slug,
        )

    return Config(
        name=name,
        dir_configs=dir_configs,
        wow_versions=wow_versions,
        curseforge_config=curseforge_config,
    )
